import * as fs from 'node:fs';
import { isCppAllowed } from './map';

export const INDENT = '    ';
export const POINTER_SIZE = 4;
export const EXTENSION = '.def';
export const STRUCTURE_FILES: {[id: string]: string} = {};
export function readDefinition(name: string) {
    if (!STRUCTURE_FILES[name]) {
        throw new Error(`Missing Definition File: ${name}${EXTENSION}`);
    }
    return fs.readFileSync(STRUCTURE_FILES[name]!, {encoding: 'utf8'});
}
export function syntaxError(message?: string) {
    throw new Error('Syntax Error' + (message ? `: ${message}` : ''));
}
export function parseTypeAndName(piece: string) {
    // Split On Last Space
    const index = piece.lastIndexOf(' ');
    if (index === -1) {
        syntaxError('Unable To Find Name/Type Divider');
    }
    let name = piece.substring(index + 1)!;
    let type = piece.substring(0, index)!;
    // Move Asterisks From Name To Type
    while (name.startsWith('*')) {
        name = name.substring(1);
        if (!type.endsWith('*')) {
            type += ' ';
        }
        type += '*';
    }
    // Return
    return {type, name};
}
export const MIN_SIZE = 1;
export function toUpperSnakeCase(str: string) {
    let wasUpper = false;
    let nextIsUpper = false;
    let out = '';
    for (let i = 0; i < str.length; i++) {
        let character = str.charAt(i);
        if (character === '_') {
            wasUpper = false;
            nextIsUpper = true;
            continue;
        }
        const isUpper = character === character.toUpperCase() || nextIsUpper;
        nextIsUpper = false;
        character = character.toUpperCase();
        if (isUpper && i > 0 && !wasUpper) {
            out += '_';
        }
        out += character;
        wasUpper = isUpper;
    }
    return out;
}
export function formatType(type: string) {
    if (!type.endsWith('*')) {
        type += ' ';
    }
    return type;
}
export const COMMENT = '//';
export function toHex(x: number) {
    return '0x' + x.toString(16);
}
export function getAssertFunction() {
    return isCppAllowed() ? 'static_assert' : '_Static_assert';
}
export function assertSize(name: string, size: number, isDefined: boolean) {
    let out = '';
    // Define Size Macro
    const macro = toUpperSnakeCase(name) + '_SIZE';
    out += `#define ${macro} ${toHex(size)}\n`;
    // Check Size
    out += `${getAssertFunction()}(sizeof (${name}) == ${macro}, "Invalid Size");\n`;
    // Hide Structure Size If The Real Size Is Unknown
    if (!isDefined) {
        out += `#undef ${macro}\n`;
    }
    // Return
    return out;
}